#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a new line art image framed (using G'MIC filters)

# (c) Dinasset 2023
#
#   History:
#
#   r0v0.0: 2023-06: Initial version
#   r0v0.1: 2023-06: added user choice of frame color  and size (by MareroQ)
#         		  + some improvements
#   r0v1.0: 2023-07: added gegl support (used for gegl-median-blur)
#   r0v2.S: 2023-07: maintained only styles which give good 
#                    improved general process and single styles
#                    added the possibiity to select the intensity of the final outcome
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import pygtk
import gtk
from sys import argv, platform
from ctypes import *
import sys, os
import time
import datetime
from gimpfu import *
############################################################################
# Service routine (for GEGL)
#---------------------------------------------------------------------------
def load_library (library_name):
    if platform == "linux" or platform == "linux2":
        library_name = library_name + '.so.0'
    elif platform == "win32":
        from ctypes.util import find_library
        library_name = find_library (library_name + "-0")
    else:
        raise BaseException ("TODO")
    return CDLL (library_name)
gimp = load_library ('libgimp-2.0')

def gegl_graph(image, drawable, gegl_graph_string):
    class GeglBuffer(Structure):
        pass
    drawable_id = drawable.ID
    
    gegl = load_library ('libgegl-0.4')
    sucess = gegl.gegl_init (None, None)
    gimp.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
    gimp.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

    x = c_int (); y = c_int (); w = c_int (); h = c_int ()
    non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
    args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
    
    if non_empty:
        source = gimp.gimp_drawable_get_buffer (drawable_id)
        target = gimp.gimp_drawable_get_shadow_buffer (drawable_id)
        sucess = gegl.gegl_render_op (source, target, "gegl:gegl", *args)
        gegl.gegl_buffer_flush (target)
        gimp.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
        gimp.gimp_drawable_update (drawable_id, x, y, w, h)
        gimp.gimp_displays_flush ()
 
def GEGL_operation_MEDIANBLUR (image, drawable, uradius):
    
    # WARNING: do not replace - with _ in the name of the GEGL function (when not simple)
    # i.e: use median-blur, not median_blur

    radius=float(uradius)
    percentile=50.0
    alpha_percentile=50.0
    mystring="gegl:median-blur radius=%f percentile=%f alpha-percentile=%f"
    gegl_graph_string=mystring % (radius, percentile, alpha_percentile)
    gegl_graph(image, drawable,gegl_graph_string)

def GEGL_operation_unsharp_mask(image, drawable, uradius, uamount):
    
    # WARNING: do not replace - with _ in the name of the GEGL function (when not simple)
    # i.e: use median-blur, not median_blur

    radius=float(uradius)
    amount=float(uamount)
    threshold=0.0
    mystring="gegl:unsharp-mask radius=%f amount=%f threshold=%f"
    gegl_graph_string=mystring % (radius, amount, threshold)
    gegl_graph(image, drawable,gegl_graph_string)
 
############################################################################
# Service routine (by MareroQ) for Frame colour
#---------------------------------------------------------------------------
def gmic_rgba_to_rgb(rgba_color):

	r = rgba_color[0]
	g = rgba_color[1]
	b = rgba_color[2]

	# write rgb in correct syntax
	rgb_color = "{0},{1},{2}".format(r,g,b)
	return rgb_color
	     
############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

############################################################################
# Main
#---------------------------------------------------------------------------

def LineartFramed_2S (inImage, inLayer,
                    frame, size_fr, color_frame,
                    style, intensity,
                    regaincolours, applymorpho, 
                    doFlatten):
     
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    
    newImage = pdb.gimp_image_duplicate (inImage)
    newBGo = newImage.layers[0]
    newBGo.name = "new BG original"
    
    # create base for applying chosen 
    baseforeffect = newBGo.copy()
    newImage.add_layer (baseforeffect, 0)
    baseforeffect.name = "base for effect"
    
    # check and -if necessary- adjust mean
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect, HISTOGRAM_VALUE,
                                                     0.0, 1.0)
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean<117.5) and (loop<100):  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.1, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean>137.5) and (loop<100):  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.9, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
      
    # added GEGL Median Blur
    radius=float(2.00)  #3.00
    percentile=50.0
    alpha_percentile=50.0
    mystring="gegl:median-blur radius=%f percentile=%f alpha-percentile=%f"
    gegl_graph_string=mystring % (radius, percentile, alpha_percentile)
    gegl_graph(newImage, baseforeffect, gegl_graph_string)
     
    # added GEGL Unsharp Mask
    radius=float(4.00)
    amount=float(2.00)
    threshold=0.0
    mystring="gegl:unsharp-mask radius=%f amount=%f threshold=%f"
    gegl_graph_string=mystring % (radius, amount, threshold)
    gegl_graph(newImage, baseforeffect, gegl_graph_string)
     
    #pdb.gimp_display_new (newImage)
    #return
     
    # Use some functionalities of G'MIC to assess structure of the image
    pdb.plug_in_gmic_qt(newImage,baseforeffect, 1,2,
        "-v - -texturize_canvas 80,4,0.6 ")
    texturized = pdb.gimp_image_get_active_layer (newImage)
    pdb.gimp_drawable_levels (texturized, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.0, 0.1, 0.5, False)   # cut DARKS and LIGHTS
    texturized.opacity=20.0
    # Name the BGlayer to use for styles
    newBG = pdb.gimp_image_merge_down (newImage, texturized, CLIP_TO_BOTTOM_LAYER)
    # cut of DARKS and LIGHTS
    pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.0, 0.1, 0.9, False)   # cut BLACKS and WHITES
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # START APPLYING USER CHOICES (Selected only those which give good results)
    
    # apply chosen GMIC Colors effect
    if style==0:    #Gradient Normalize
        # Apply gradient_norm
        #[G'MIC] fx_gradient_norm 0,0.5,0,60,0,0,50,50
        parms='0,0.5,0,60,0,0,50,50'    
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_gradient_norm "+str(parms)+" ")    
        
        LineartLayer.name = "Lineart Gradient Norm "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
                                           
    #pdb.gimp_display_new (newImage)
    #return
         
    if style==1:    #Edge (by afre)
        # Apply afre_edge
        #[G'MIC] -afre_edge 0,1,1,1,1 #>>> 0,1,4,4,1 
        parms='0,1,4,4,1'
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -afre_edge "+str(parms))     
        
        LineartLayer.name = "Lineart afre_edge "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
        tempE=LineartLayer.copy()
        newImage.add_layer (tempE, 0)
        pdb.gimp_drawable_equalize (tempE,False) # reqd, to adjust values too light
        tempE.opacity=50.0
        tempE.mode = LAYER_MODE_VIVID_LIGHT
        LineartLayer = pdb.gimp_image_merge_down (newImage, tempE,
                                                  CLIP_TO_BOTTOM_LAYER)
    
    if style==2:    #Edges
        # Apply fx_edges
        #[G'MIC] fx_edges 0,10,0,0,50,50
        parms='0,4.5,0,0,50,50'
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_edges "+str(parms))
        
        LineartLayer.name = "Lineart edges Layer "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_drawable_invert (LineartLayer,False)    # already inverted
        
    if style==3:    #JR_Gradient_Norm
        # Apply jr_gradient_norm
        #[G'MIC] fx_jr_gradient_norm 0,0.45,10,50,1,0,1,1,2,0,0,50,50
        parms='0,0.45,10,50,1,0,1,1,2,0,0,50,50 '
    
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
        

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_jr_gradient_norm "+str(parms))
           
        LineartLayer.name = "Lineart JR Grad Norm "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_drawable_invert (LineartLayer,False)
        
    #pdb.gimp_display_new (newImage)
    #return
        
    if style==4:    #Thin_Edges
        # Apply thin_edges
        #[G'MIC] fx_thin_edges 0,9.5,0,0,50,50
        parms='0,9.5,0,0,50,50'
        
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)
        
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_thin_edges "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart: Thin_Edges "+str(parms)        
        
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #already inverted
    
    # -----------------------------------------------------------------------------
    # CONVOLVE: various types
    # -----------------------------------------------------------------------------

    if style==5:    #Convolve (Laplacian)
        # Apply convolve
        # [G'MIC] fx_convolve 11,1,"0,1,0;1,-4,1;0,1,0",0,5,2,0,50,50  
        parms='11,1,"0,1,0;1,-4,1;0,1,0",0,5,2,0,50,50' #test
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Laplacian "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        pdb.gimp_drawable_invert (LineartLayer,False)
        
        tempE=LineartLayer.copy()
        newImage.add_layer (tempE, 0)
        pdb.gimp_drawable_equalize (tempE,False) # reqd, to adjust values too light
        tempE.opacity=90.0
        tempE.mode = LAYER_MODE_VIVID_LIGHT
        LineartLayer = pdb.gimp_image_merge_down (newImage, tempE,
                                                  CLIP_TO_BOTTOM_LAYER)
        
    # ------------------------------
    # CONVOLVE: various types double
    # ------------------------------
                        
    if style==6:    #Convolve (Prewitt X+Y)
        # Apply Prewitt X
        #[G'MIC] fx_convolve 5,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50
        parms='5,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'  
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Prewitt X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        
        # Apply Prewitt Y
        # [G'MIC] fx_convolve 6,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50  
        parms='6,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'  
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Prewitt Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (newImage)
        #return
        
        parms='5&6,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Prewitt X+Y "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
        
    if style==7:    #Convolve (Sobel X+Y)
        # Apply Sobel X
        # [G'MIC] fx_convolve 7,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50  
        parms='7,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Sobel X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        # Apply Sobel Y
        #[G'MIC] fx_convolve 8,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50   
        parms='8,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Sobel Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (newImage)
        #return
        
        parms='7&8,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Sobel X+Y "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_display_new (newImage)
        #return
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
    if style==8:    #Convolve (Potinv X+Y)
        # Apply Potinv X
        # [G'MIC] fx_convolve 9,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50  
        parms='9,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50'
   
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Potinv X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        # Apply Potinv Y
        # [G'MIC] fx_convolve 10,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50   
        parms='10,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Potinv Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (newImage)
        #return
        
        parms='9&10,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Potinv X+Y "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0

        pdb.gimp_drawable_invert (LineartLayer,False)    
        
    if style==9:    #Convolve (Cross 1+2)
        # Apply Cross 1
        # [G'MIC] fx_convolve 12,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50 
         
        parms='12,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50'
   
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Cross 1 "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        # Apply Cross 2
        #[G'MIC]  fx_convolve 13,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50 
        parms='13,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        newImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Cross 2 "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 50.0
              
        #pdb.gimp_display_new (newImage)
        #return
        
        parms='12&13,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (newImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Cross 1+2 "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_display_new (newImage)
        #return
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
    if style==10:    #Isophotes #new
        # Apply isophotes
        # [G'MIC] fx_isophotes 31,0.75,1,0,50,50    
        parms='31,0.75,1,0,50,50'
  
        LineartLayer = newBG.copy()
        newImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_isophotes "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Isophotes "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
    ##################################################################
    
    # COMMON ******************************************************
    
    # Style layer  
    
    StyleLayer = pdb.gimp_layer_new_from_visible (newImage,
                                                  newImage,
                                                  "Visible layer for Style")
    newImage.add_layer (StyleLayer, 0)
       
    # APPLY LEVELS
    pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                                5.0, 0.1, 0.9, False)
    
    StyleLayer.mode = LAYER_MODE_BURN   #MULTIPLY #NORMAL 
    StyleLayer.opacity = 90.0   #100.0
        
    #pdb.gimp_display_new (newImage)
    #return
     
    # added GEGL Median Blur
    radius=float(0.50)
    percentile=50.0
    alpha_percentile=50.0
    gegl_graph_string="gegl:median-blur radius=%f percentile=%f alpha-percentile=%f" % (radius, percentile, alpha_percentile)
    gegl_graph(newImage, StyleLayer, gegl_graph_string)
     
    # added BG desaturated on top
    BGdesat = newBGo.copy()
    newImage.add_layer (BGdesat, 0)
    pdb.gimp_drawable_desaturate (BGdesat, DESATURATE_LUMINANCE)
    if style==10:    #Isophotes #new
        BGdesat.mode = LAYER_MODE_SCREEN
    else:    
        BGdesat.mode = LAYER_MODE_HARDLIGHT #SOFTLIGHT #OVERLAY #BURN #LINEAR_BURN   
    BGdesat.name = "Original desaturated"
    BGdesat.opacity = 90.0  #100.0
    # APPLY LEVELS on Original desaturated
    pdb.gimp_drawable_levels (BGdesat, HISTOGRAM_VALUE,
                                  0.0, 1.0, False,
                                  1.0, 0.1, 0.9, False)
    
    prefinal_layer = pdb.gimp_layer_new_from_visible (newImage, newImage,
                                                      "Pre_Outcome")
    newImage.add_layer (prefinal_layer, 0)
    prefinal_layer.name = "Preliminary Outcome"
    prefinal_layer.mode = LAYER_MODE_NORMAL
        
    #pdb.gimp_display_new (newImage)
    #return
    
    if applymorpho>0:
        # bigger size, to minimize the effect
        pdb.gimp_layer_scale (prefinal_layer,
                              prefinal_layer.width*4/3,
                              prefinal_layer.height*4/3,
                              True)
        if applymorpho==1:
            # apply morpho-dilation
            # fx_morphological 1,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50
            parms='1,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50'
        if applymorpho==2:
            # apply morpho-erosion
            # fx_morphological 0,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50
            parms='0,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,LineartLayer, 1,0,
        "-v - -fx_morphological "+str(parms)+" ")     
        # back to original size
        pdb.gimp_layer_scale (prefinal_layer,
                              newBG.width,
                              newBG.height,
                              True)
    if regaincolours==True:
        # RE-APPLY LEVELS on PreFinal
        pdb.gimp_drawable_levels (prefinal_layer, HISTOGRAM_VALUE,
                                  0.0, 1.0, False,
                                  1.0, 0.1, 0.9, False)
        BGonTop = newBGo.copy()
        newImage.add_layer (BGonTop, 0)
        BGonTop.mode = LAYER_MODE_LCH_COLOR
        BGonTop.opacity = 100.0
    else:
        pdb.gimp_drawable_desaturate (prefinal_layer, DESATURATE_LUMINANCE)
        
    #pdb.gimp_display_new (newImage)
    #return
     
    # outcome
    
    outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
    newImage.add_layer (outcome, 0)
    outcome.name = "Outcome (unframed)"
        
    #pdb.gimp_display_new (newImage)
    #return
    
    # G'MIC Auto Balance outcome
    # gcd_auto_balance 30,0,0,1,0
    outcomeb = outcome.copy()
    newImage.add_layer (outcomeb, 0)
    parms='30,0,0,1,0'
    pdb.plug_in_gmic_qt(newImage,outcomeb, 1,0,
        "-v - -gcd_auto_balance "+str(parms)+" ")
    outcomeb.opacity = 30.0
    outcome=pdb.gimp_image_merge_down (newImage,outcomeb,0)
    
    # apply GMIC Frame Blur  
    # fx_frame_blur 10,10,0,5,1,0,128,128,128,0,0,255,255,255,2,2,1,0,0.5,0.5,0
    hex = gmic_rgba_to_rgb(color_frame)
    SH = int(size_fr)
    SV = int(size_fr)
    Round=["0.0","1.0","0.5","0.2","0.1"]	
    Rnd=str(Round[frame])
    XSh = min(10,int(size_fr/6))
    YSh = min(10,int(size_fr/6))
    OuS = min(10,int(size_fr/3))	
    parms=str(SH)+','+str(SV)+',0,5,'+str(Rnd)+',0,128,128,128,0,'
    parms=parms+str(OuS)+','+str(hex)+','+str(XSh)+','+str(YSh)+',1,0,0.5,0.5,0 '	
    pdb.plug_in_gmic_qt(newImage, outcome, 1,2,
        "-v - -fx_frame_blur "+str(parms)+" ")
    Framed = pdb.gimp_image_get_active_layer(newImage)
    Framed.mode = LAYER_MODE_NORMAL
 
    if doFlatten==True:
        pdb.gimp_image_remove_layer (newImage, outcome)
        if regaincolours==True:
            pdb.gimp_image_remove_layer (newImage, BGonTop)
        pdb.gimp_image_remove_layer (newImage, prefinal_layer)
        #pdb.gimp_image_remove_layer (newImage, Iso_layer)
        pdb.gimp_image_remove_layer (newImage, BGdesat)
        pdb.gimp_image_remove_layer (newImage, StyleLayer)
        pdb.gimp_image_remove_layer (newImage, LineartLayer)
        pdb.gimp_image_remove_layer (newImage, newBG)
         
    #pdb.gimp_display_new (newImage)
     
    stylename = ["Gradient Normalize","Afre Edge",
                 "Edges","Gradient Norm JR",
                 "Thin_Edges",
                 "Convolve Prewitt X","Convolve Prewitt Y",
                 "Convolve Sobel X","Convolve Sobel Y",
                 "Convolve Potinv X","Convolve Potinv Y",
                 "Convolve Laplacian",
                 "Convolve Cross 1","Convolve Cross 2",
                 "Convolve Prewitt X+Y",
                 "Convolve Sobel X+Y",
                 "Convolve Potinv X+Y",
                 "Convolve Cross 1+2",
                 "Isophotes",
                  ]     
    Framed.name = st+oname+" with a framed line art v.1.2 effect, style "
    Framed.name = Framed.name+" "+str(stylename[style])
    if regaincolours==True:
        Framed.name = Framed.name+" (coloured)"
    if applymorpho==1:
        Framed.name = Framed.name+" (reducing black strokes)"
    elif applymorpho==2:
        Framed.name = Framed.name+" (increasing black strokes)"
             
    Framed2 = Framed.copy()
    newImage.add_layer (Framed2, 0)
    Framed2.mode = LAYER_MODE_LINEAR_BURN
    if intensity==0:
        Framed2.opacity=0.0
    elif intensity==1:
        Framed2.opacity=25.0
    elif intensity==2:
        Framed2.opacity=50.0
    elif intensity==3:
        Framed2.opacity=75.0
    else:
        Framed2.opacity = 100.0
    Framed=pdb.gimp_image_merge_down (newImage,Framed2,0)
    Framed.mode = LAYER_MODE_NORMAL
    Framed.opacity = 100.0
        
    pdb.gimp_display_new (newImage)
    return
             
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleFilters/"
copyrightYear="2023"
desc="Line Art Framed v.1.2S"

register(
    "LineartFramed_2S",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Line Art Framed v.1.2S ...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_OPTION, "frame", "Choose frame format", 0,
                     ["square","round","half round",
                      "0.2 rounded square","0.1 rounded square"
                      ]),
        (PF_SPINNER, "size_fr",    "Frame thickness", 20, (0, 50, 2)),					  
        (PF_COLOR,   "color_frame", "Color frame", (200,200,200)),					  
        (PF_OPTION, "baseeffect", "Choose GMIC base effect ", 0,
                     ["Gradient Normalize","Afre Edge",
                      "Edges","Gradient Norm JR",
                      "Thin_Edges (very slow)",
                      "Convolve Laplacian",
                      "Convolve Prewitt X+Y",
                      "Convolve Sobel X+Y",
                      "Convolve Potinv X+Y",
                      "Convolve Cross 1+2",
                      "Isophotes",					  
                      ]),
        (PF_OPTION, "flook", "Choose final look intensity", 2,
                     ["*base","delicate","average",
                      "intense","very intense",
                      ]),
        (PF_TOGGLE, "getcolours", "Regain colours for final outcome?", False),
        (PF_OPTION, "morpho", "Apply morphological filter for final outcome?",
                                0,["*none","Dilate","Erode"]),
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    LineartFramed_2S,
    menu=menu
)

main()
